﻿//-----------------------------------------------------------------------
// <copyright>
// This file is subject to the terms and conditions of the Microsoft Public License (MS-PL).
// See http://www.microsoft.com/opensource/licenses.mspx#Ms-PL for more details.
// </copyright>
//----------------------------------------------------------------------- 
using Fulcrum.ControlGenerator;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Fulcrum.UnitTests.ControlGenerator
{
    [TestClass]
    public class RoutedCommandXmlConfigurationTests
    {
        public RoutedCommandXmlConfigurationTests()
        {
            // nothing
        }

        const string StandardXml = "<blah name='my name' canExecuteAlways='true' summary='my summary' ></blah>";
        const string EmptyNameXml = "<blah name='' canExecuteAlways='true' summary='my summary' ></blah>";
        const string MissingNameXml = "<blah canExecuteAlways='true' summary='my summary' ></blah>";
        const string EmptyCanExecuteAlwaysXml = "<blah name='my name' canExecuteAlways='' summary='my summary' ></blah>";
        const string MissingCanExecuteAlwaysXml = "<blah name='my name' summary='my summary' ></blah>";
        const string FalseCanExecuteAlwaysXml = "<blah name='my name' canExecuteAlways='false' summary='my summary' ></blah>";
        const string EmptySummaryXml = "<blah name='my name' canExecuteAlways='true' summary='' ></blah>";
        const string MissingSummaryXml = "<blah name='my name' canExecuteAlways='true' ></blah>";

        [TestMethod]
        public void TestReaderAdvancement()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(StandardXml);

            Assert.IsTrue(xmlReader.IsStartElement());

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);

            Assert.IsFalse(xmlReader.IsStartElement());
        }

        [TestMethod]
        public void TestCanExecuteAlways()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(StandardXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.IsTrue(config.CanExecuteAlways);
        }

        private static RoutedCommandXmlConfiguration CreateRoutedCommandXmlConfiguration(System.Xml.XmlReader xmlReader)
        {
            var controlConfig = new MockControlConfiguration();
            controlConfig.Name = "MyControl";

            var config = new RoutedCommandXmlConfiguration(controlConfig, xmlReader);
            return config;
        }

        [TestMethod]
        public void TestMissingCanExecuteAlways()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(MissingCanExecuteAlwaysXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.IsTrue(config.CanExecuteAlways);
        }

        [TestMethod]
        public void TestEmptyCanExecuteAlways()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(EmptyCanExecuteAlwaysXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.IsTrue(config.CanExecuteAlways);
        }

        [TestMethod]
        public void TestFalseCanExecuteAlways()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(FalseCanExecuteAlwaysXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.IsFalse(config.CanExecuteAlways);
        }

        [TestMethod]
        public void TestName()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(StandardXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.AreEqual("my name", config.Name);
        }

        [TestMethod]
        public void TestMissingName()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(MissingNameXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.IsNull(config.Name);
        }

        [TestMethod]
        public void TestEmptyName()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(EmptyNameXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.AreEqual("", config.Name);
        }

        [TestMethod]
        public void TestOwningType()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(StandardXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.AreEqual("MyControl", config.OwningType);
        }

        [TestMethod]
        public void TestSummary()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(StandardXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.AreEqual("my summary", config.Summary);
        }

        [TestMethod]
        public void TestMissingSummary()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(MissingSummaryXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.IsNull(config.Summary);
        }

        [TestMethod]
        public void TestEmptySummary()
        {
            var xmlReader = XmlTestHelp.CreateXmlReader(EmptySummaryXml);

            var config = CreateRoutedCommandXmlConfiguration(xmlReader);
            Assert.AreEqual("", config.Summary);
        }

    }
}
